@@ -99,6 +99,8 @@ class GXDLMSClient(object):
         self.isAuthenticationRequired = False
         # Auto increase Invoke ID.
         self.autoIncreaseInvokeID = False
+        #If protected release is used release is including a ciphered xDLMS Initiate request.
+        self.useProtectedRelease = False
 
     def __getObjects(self):
         return self.settings.objects
@@ -249,16 +251,23 @@ class GXDLMSClient(object):
         self.settings.useUtc2NormalTime = value
 
     #
-    # Standard says that Time zone is from normal time to UTC in
-    #      minutes.  If
-    # meter is configured to use UTC time (UTC to normal time) set this
-    #      to
-    # true.
+    # Standard says that Time zone is from normal time to UTC in minutes.
+    # If meter is configured to use UTC time (UTC to normal time) set this to true.
     #
     # True, if UTC time is used.
     #
     useUtc2NormalTime = property(__getUseUtc2NormalTime, __setUseUtc2NormalTime)
 
+    def __getDateTimeSkips(self):
+        return self.settings.dateTimeSkips
+
+    def __setDateTimeSkips(self, value):
+        self.settings.dateTimeSkips = value
+
+    dateTimeSkips = property(__getDateTimeSkips, __setDateTimeSkips)
+    """Skipped date time fields. This value can be used if meter can't handle deviation or status."""
+
+
     def __getStandard(self):
         return self.settings.standard
 
@@ -491,7 +500,8 @@ class GXDLMSClient(object):
             self.settings.setInvokeID(1)
         #  If authentication or ciphering is used.
         if self.authentication > Authentication.LOW:
-            self.settings.ctoSChallenge = GXSecure.generateChallenge()
+            if not self.settings.useCustomChallenge:
+                self.settings.ctoSChallenge = GXSecure.generateChallenge()
         else:
             self.settings.setCtoSChallenge(None)
         _GXAPDU.generateAarq(self.settings, self.settings.cipher, None, buff)
@@ -609,16 +619,16 @@ class GXDLMSClient(object):
         if (self.settings.connected & ConnectionState.DLMS) == 0:
             return None
         buff = GXByteBuffer()
-        buff.setUInt8(0)
+        buff.setUInt8(3)
         buff.setUInt8(0x80)
         buff.setUInt8(1)
         buff.setUInt8(00)
-        #Increase IC.
-        if self.settings.cipher and self.settings.cipher.isCiphered:
-            self.settings.cipher.invocationCounter = self.settings.cipher.invocationCounter + 1
-        _GXAPDU.generateUserInformation(self.settings, self.settings.cipher, None, buff)
-        buff.setUInt8(len(buff) - 1, 0)
-        reply = None
+        if self.useProtectedRelease:
+            #Increase IC.
+            if self.settings.cipher and self.settings.cipher.isCiphered:
+                self.settings.cipher.invocationCounter = self.settings.cipher.invocationCounter + 1
+            _GXAPDU.generateUserInformation(self.settings, self.settings.cipher, None, buff)
+            buff.setUInt8(len(buff) - 1, 0)
         if self.useLogicalNameReferencing:
             p = GXDLMSLNParameters(self.settings, 0, Command.RELEASE_REQUEST, 0, buff, None, 0xff)
             reply = GXDLMS.getLnMessages(p)
@@ -1053,8 +1063,6 @@ class GXDLMSClient(object):
         sort = pg.sortObject
         if not sort and pg.captureObjects:
             sort = pg.captureObjects[0][0]
-        if not sort or sort.objectType != ObjectType.CLOCK:
-            return self.read(pg, 2)
         buff = GXByteBuffer(51)
         buff.setUInt8(0x01)
         buff.setUInt8(DataType.STRUCTURE)
@@ -1065,7 +1073,7 @@ class GXDLMSClient(object):
         _GXCommon.setData(self.settings, buff, DataType.OCTET_STRING, _GXCommon.logicalNameToBytes(sort.logicalName))
         _GXCommon.setData(self.settings, buff, DataType.INT8, 2)
         _GXCommon.setData(self.settings, buff, DataType.UINT16, sort.version)
-        if pg.captureObjects and isinstance(pg.captureObjects[0][0], GXDLMSData) and pg.captureObjects[0][0].logicalName == "0.0.1.1.0.255":
+        if sort and isinstance(sort, GXDLMSData) and sort.logicalName == "0.0.1.1.0.255":
             _GXCommon.setData(self.settings, buff, DataType.UINT32, GXDateTime.toUnixTime(start))
             _GXCommon.setData(self.settings, buff, DataType.UINT32, GXDateTime.toUnixTime(end))
         else:
