@@ -56,6 +56,8 @@ from ..GXUInt64 import GXUInt64
 from ..GXDateTime import GXDateTime
 from ..GXDate import GXDate
 from ..GXTime import GXTime
+from ..GXFloat32 import GXFloat32
+from ..GXFloat64 import GXFloat64
 
 # pylint: disable=too-many-public-methods
 class _GXCommon:
@@ -632,7 +634,7 @@ class _GXCommon:
             tmp = GXByteBuffer()
             cls.setData(settings, tmp, DataType.FLOAT64, value)
             info.xml.appendLine(info.xml.getDataType(info.type_), None, GXByteBuffer.toHex(False, 1, len(tmp) - 1))
-        return value
+        return GXFloat64(value)
 
     #
     # Get float value from DLMS data.
@@ -657,7 +659,7 @@ class _GXCommon:
             tmp = GXByteBuffer()
             cls.setData(settings, tmp, DataType.FLOAT32, value)
             info.xml.appendLine(info.xml.getDataType(info.type_), None, tmp.toHex(False, 1, len(tmp) - 1))
-        return value
+        return GXFloat32(value)
 
     #
     # Get enumeration value from DLMS data.
@@ -1032,6 +1034,7 @@ class _GXCommon:
                 return None
         if len_ > 0:
             value = buff.getString(buff.position, len_)
+            buff.position += len_
         else:
             value = ""
         if info.xml:
@@ -1422,8 +1425,12 @@ class _GXCommon:
     @classmethod
     def setDateTime(cls, settings, buff, value):
         dt = cls.__getDateTime(value)
+        skip = dt.skip
+        if settings and settings.dateTimeSkips:
+            skip = skip or settings.dateTimeSkips
+
         #  Add year.
-        if dt.skip & DateTimeSkips.YEAR != DateTimeSkips.NONE:
+        if skip & DateTimeSkips.YEAR != DateTimeSkips.NONE:
             buff.setUInt16(0xFFFF)
         else:
             buff.setUInt16(dt.value.year)
@@ -1432,7 +1439,7 @@ class _GXCommon:
             buff.setUInt8(0xFD)
         elif dt.extra & DateTimeExtraInfo.DST_END != 0:
             buff.setUInt8(0xFE)
-        elif dt.skip & DateTimeSkips.MONTH != DateTimeSkips.NONE:
+        elif skip & DateTimeSkips.MONTH != DateTimeSkips.NONE:
             buff.setUInt8(0xFF)
         else:
             buff.setUInt8(dt.value.month)
@@ -1442,12 +1449,12 @@ class _GXCommon:
             buff.setUInt8(0xFD)
         elif dt.extra & DateTimeExtraInfo.LAST_DAY != DateTimeSkips.NONE:
             buff.setUInt8(0xFE)
-        elif dt.skip & DateTimeSkips.DAY != DateTimeSkips.NONE:
+        elif skip & DateTimeSkips.DAY != DateTimeSkips.NONE:
             buff.setUInt8(0xFF)
         else:
             buff.setUInt8(dt.value.day)
         #  Day of week.
-        if dt.skip & DateTimeSkips.DAY_OF_WEEK != DateTimeSkips.NONE:
+        if skip & DateTimeSkips.DAY_OF_WEEK != DateTimeSkips.NONE:
             buff.setUInt8(0xFF)
         else:
             if dt.dayOfWeek == 0:
@@ -1455,19 +1462,19 @@ class _GXCommon:
             else:
                 buff.setUInt8(dt.dayOfWeek)
         #  Add time.
-        if dt.skip & DateTimeSkips.HOUR != DateTimeSkips.NONE:
+        if skip & DateTimeSkips.HOUR != DateTimeSkips.NONE:
             buff.setUInt8(0xFF)
         else:
             buff.setUInt8(dt.value.hour)
-        if dt.skip & DateTimeSkips.MINUTE != DateTimeSkips.NONE:
+        if skip & DateTimeSkips.MINUTE != DateTimeSkips.NONE:
             buff.setUInt8(0xFF)
         else:
             buff.setUInt8(dt.value.minute)
-        if dt.skip & DateTimeSkips.SECOND != DateTimeSkips.NONE:
+        if skip & DateTimeSkips.SECOND != DateTimeSkips.NONE:
             buff.setUInt8(0xFF)
         else:
             buff.setUInt8(dt.value.second)
-        if dt.skip & DateTimeSkips.MILLISECOND != DateTimeSkips.NONE:
+        if skip & DateTimeSkips.MILLISECOND != DateTimeSkips.NONE:
             #  Hundredth of seconds is not used.
             buff.setUInt8(0xFF)
         else:
@@ -1476,7 +1483,7 @@ class _GXCommon:
                 ms /= 10000
             buff.setUInt8(int(ms))
         #  devitation not used.
-        if dt.skip & DateTimeSkips.DEVITATION != DateTimeSkips.NONE:
+        if skip & DateTimeSkips.DEVITATION != DateTimeSkips.NONE:
             buff.setUInt16(0x8000)
         else:
             #  Add devitation.
@@ -1485,7 +1492,7 @@ class _GXCommon:
                 d = -d
             buff.setUInt16(d)
         #  Add clock_status
-        if dt.skip & DateTimeSkips.STATUS == DateTimeSkips.NONE:
+        if skip & DateTimeSkips.STATUS == DateTimeSkips.NONE:
             if dt.value.dst() or dt.status & ClockStatus.DAYLIGHT_SAVE_ACTIVE != ClockStatus.OK:
                 buff.setUInt8(dt.status | ClockStatus.DAYLIGHT_SAVE_ACTIVE)
             else:
@@ -1584,8 +1591,7 @@ class _GXCommon:
         elif value == DataType.OCTET_STRING:
             ret = bytes.__class__
         elif value == DataType.ENUM:
-            ret = int.__class__
-            #ret = Enum.__class__
+            ret = GXEnum.__class__
         elif value == DataType.INT8:
             ret = int.__class__
         elif value == DataType.INT16:
@@ -1594,6 +1600,14 @@ class _GXCommon:
             ret = int.__class__
         elif value == DataType.INT64:
             ret = int.__class__
+        elif value == DataType.UINT8:
+            ret = GXUInt8.__class__
+        elif value == DataType.UINT16:
+            ret = GXUInt16.__class__
+        elif value == DataType.UINT32:
+            ret = GXUInt32.__class__
+        elif value == DataType.UINT64:
+            ret = GXUInt64.__class__
         elif value == DataType.TIME:
             ret = GXTime.__class__
         elif value == DataType.DATE:
@@ -1607,16 +1621,18 @@ class _GXCommon:
         elif value == DataType.BOOLEAN:
             ret = bool.__class__
         elif value == DataType.FLOAT32:
-            ret = float.__class__
+            ret = GXFloat32.__class__
         elif value == DataType.FLOAT64:
-            ret = float.__class__
+            ret = GXFloat64.__class__
+        elif value == DataType.BITSTRING:
+            ret = GXBitString.__class__
         else:
             raise ValueError("Invalid value.")
         return ret
 
     @classmethod
     def getDLMSDataType(cls, value):
-       # pylint: disable=undefined-variable
+        # pylint: disable=undefined-variable
         if value is None:
             ret = DataType.NONE
         elif isinstance(value, (bytes, bytearray, GXByteBuffer)):
@@ -1637,30 +1653,39 @@ class _GXCommon:
             ret = DataType.UINT32
         elif isinstance(value, (GXUInt64)):
             ret = DataType.UINT64
-        elif isinstance(value, bool):
+        elif isinstance(value, (bool)):
             ret = DataType.BOOLEAN
         elif isinstance(value, (GXInt32, int)):
             ret = DataType.INT32
-        elif isinstance(value, GXTime):
+        elif isinstance(value, (GXTime)):
             ret = DataType.TIME
-        elif isinstance(value, GXDate):
+        elif isinstance(value, (GXDate)):
             ret = DataType.DATE
         elif isinstance(value, (datetime, GXDateTime)):
             ret = DataType.DATETIME
-        elif isinstance(value, GXStructure):
+        elif isinstance(value, (GXStructure)):
             ret = DataType.STRUCTURE
         elif isinstance(value, (GXArray, list)):
             ret = DataType.ARRAY
-        elif isinstance(value, str):
+        elif isinstance(value, (str)):
             ret = DataType.STRING
-        elif isinstance(value, float):
+        elif isinstance(value, (GXFloat64)):
+            ret = DataType.FLOAT64
+        elif isinstance(value, (GXFloat32, complex, float)):
             ret = DataType.FLOAT32
-        elif isinstance(value, GXBitString):
+        elif isinstance(value, (GXBitString)):
             ret = DataType.BITSTRING
-        elif isinstance(value, unicode):
-            ret = DataType.STRING
         else:
-            raise ValueError("Invalid datatype " + type(value) + ".")
+            ret = None
+        if ret is None:
+            #Python 2.7 uses unicode.
+            try:
+                if isinstance(value, (unicode)):
+                    ret = DataType.STRING
+            except Exception:
+                ret = None
+            if ret is None:
+                raise ValueError("Invalid datatype " + type(value) + ".")
         return ret
 
     @classmethod
@@ -1858,3 +1883,14 @@ class _GXCommon:
             not cls.__getChar(st[6]).isdigit() or not cls.__getChar(st[7]).isdigit():
             return cls.idisSystemTitleToString(st)
         return cls.dlmsSystemTitleToString(st)
+
+    #Reserved for internal use.
+    @classmethod
+    def swapBits(cls, value):
+        ret = 0
+        pos = 0
+        while pos != 8:
+            ret = ret << 1 or value & 0x01
+            value = value >> 1
+            pos = pos + 1
+        return ret
