@@ -35,13 +35,13 @@ from .GXDLMSObject import GXDLMSObject
 from .IGXDLMSBase import IGXDLMSBase
 from ..enums import ErrorCode
 from ..internal._GXCommon import _GXCommon
+from .enums import Weekdays
 from ..enums import ObjectType, DataType
 from .GXDLMSScheduleEntry import GXDLMSScheduleEntry
 from ..GXDate import GXDate
 from ..GXTime import GXTime
 from ..GXByteBuffer import GXByteBuffer
 from ..GXBitString import GXBitString
-from ..GXUInt8 import GXUInt8
 
 # pylint: disable=too-many-instance-attributes
 class GXDLMSSchedule(GXDLMSObject, IGXDLMSBase):
@@ -187,8 +187,7 @@ class GXDLMSSchedule(GXDLMSObject, IGXDLMSBase):
         data.setUInt8(DataType.UINT16)
         data.setUInt16(it.validityWindow)
         #Add exec week days.
-        bs = GXBitString(GXUInt8(it.execWeekdays), 7)
-        _GXCommon.setData(None, data, DataType.BITSTRING, bs)
+        _GXCommon.setData(None, data, DataType.BITSTRING, GXBitString.toBitString(it.execWeekdays, 7))
         #Add exec spec days.
         _GXCommon.setData(None, data, DataType.BITSTRING, it.execSpecDays)
         #Add begin date.
@@ -216,18 +215,18 @@ class GXDLMSSchedule(GXDLMSObject, IGXDLMSBase):
     # Create a new entry.
     #
     @classmethod
-    def createEntry(cls, it):
+    def createEntry(cls, settings, it):
         item = GXDLMSScheduleEntry()
         item.index = it[0]
         item.enable = it[1]
         item.logicalName = _GXCommon.toLogicalName(it[2])
         item.scriptSelector = it[3]
-        item.switchTime = _GXCommon.changeType(None, it[4], DataType.TIME)
+        item.switchTime = _GXCommon.changeType(settings, it[4], DataType.TIME)
         item.validityWindow = it[5]
-        item.execWeekdays = it[6].toByte()
-        item.execSpecDays = it[7]
-        item.beginDate = _GXCommon.changeType(None, it[8], DataType.DATE)
-        item.endDate = _GXCommon.changeType(None, it[9], DataType.DATE)
+        item.execWeekdays = Weekdays(it[6].toInteger())
+        item.execSpecDays = str(it[7])
+        item.beginDate = _GXCommon.changeType(settings, it[8], DataType.DATE)
+        item.endDate = _GXCommon.changeType(settings, it[9], DataType.DATE)
         return item
 
     #
@@ -240,7 +239,7 @@ class GXDLMSSchedule(GXDLMSObject, IGXDLMSBase):
             self.entries = []
             if e.value:
                 for it in e.value:
-                    self.entries.append(self.createEntry(it))
+                    self.entries.append(self.createEntry(settings, it))
         else:
             e.error = ErrorCode.READ_WRITE_DENIED
 
@@ -255,7 +254,7 @@ class GXDLMSSchedule(GXDLMSObject, IGXDLMSBase):
                 it.scriptSelector = reader.readElementContentAsInt("ScriptSelector")
                 it.switchTime = reader.readElementContentAsTime("SwitchTime")
                 it.validityWindow = reader.readElementContentAsInt("ValidityWindow")
-                it.execWeekdays = reader.readElementContentAsString("ExecWeekdays")
+                it.execWeekdays = Weekdays(reader.readElementContentAsInt("ExecWeekdays"))
                 it.execSpecDays = reader.readElementContentAsString("ExecSpecDays")
                 it.beginDate = reader.readElementContentAsDate("BeginDate")
                 it.endDate = reader.readElementContentAsDate("EndDate")
@@ -273,7 +272,7 @@ class GXDLMSSchedule(GXDLMSObject, IGXDLMSBase):
                 writer.writeElementString("ScriptSelector", it.scriptSelector)
                 writer.writeElementString("SwitchTime", it.switchTime)
                 writer.writeElementString("ValidityWindow", it.validityWindow)
-                writer.writeElementString("ExecWeekdays", it.execWeekdays)
+                writer.writeElementString("ExecWeekdays", int(it.execWeekdays))
                 writer.writeElementString("ExecSpecDays", it.execSpecDays)
                 writer.writeElementString("BeginDate", it.beginDate)
                 writer.writeElementString("EndDate", it.endDate)
